<?php
// $Id: ds.view_modes.inc,v 1.1.2.8 2011/02/03 17:34:59 swentel Exp $

/**
 * @file
 * Administrative functions for managing custom view modes for every entity.
 */

/**
 * Shows the list of custom view modes.
 */
function ds_custom_view_modes_list() {
  $output = '';

  $custom_view_modes = ds_custom_view_modes();
  if (!empty($custom_view_modes)) {

    $rows = array();
    foreach ($custom_view_modes as $view_mode_key => $view_mode_value) {
      $row = array();
      $row[] = check_plain($view_mode_value->label);
      $row[] = $view_mode_key;
      $row[] = ucwords(str_replace('_', ' ', implode(', ', $view_mode_value->entities)));
      $operations = l(t('Edit'), 'admin/structure/ds/view_modes/manage/' . $view_mode_key, array('alias' => TRUE));
      $operations .= ' - ' . l(t('Delete'), 'admin/structure/ds/view_modes/delete/' . $view_mode_key, array('alias' => TRUE));
      $row[] = $operations;
      $rows[] = $row;
    }

    $table = array(
      'header' => array(
        'Label',
        'Machine name',
        'Entities',
        'Operations',
      ),
      'rows' => $rows,
    );

    $output = theme('table', $table);
  }
  else {
    $output = t('No custom view modes have been defined.');
  }

  return $output;
}

/**
 * Manage a custom view mode.
 */
function ds_edit_view_mode_form($form, &$form_state, $view_mode = '') {

  $custom_view_modes = ds_custom_view_modes();
  if (isset($custom_view_modes[$view_mode])) {
    $view_mode = $custom_view_modes[$view_mode];
  }
  else {
    $view_mode = new stdClass;
    $view_mode->label = '';
    $view_mode->view_mode = '';
    $view_mode->entities = array();
  }

  $form['name'] = array(
    '#title' => t('Label'),
    '#type' => 'textfield',
    '#default_value' => $view_mode->label,
    '#description' => t('The human-readable label of the view mode. This name must be unique.'),
    '#required' => TRUE,
    '#size' => 30,
  );

  $form['view_mode'] = array(
    '#title' => t('Machine name'),
    '#type' => 'machine_name',
    '#default_value' => $view_mode->view_mode,
    '#maxlength' => 32,
    '#description' => t('The machine-readable name of this view mode. This name must contain only lowercase letters and underscores. This name must be unique.'),
    '#disabled' => !empty($view_mode->view_mode),
    '#machine_name' => array(
      'exists' => 'ds_view_mode_unique',
    ),
  );

  $entity_options = array();
  $entities = entity_get_info();
  foreach ($entities as $entity_type => $entity_info) {
    if ($entity_info['fieldable']) {
      $entity_options[$entity_type] = drupal_ucfirst(str_replace('_', ' ', $entity_type));
    }
  }
  $form['entities'] = array(
    '#title' => t('Entities'),
    '#description' => t('Select the entities for which this view mode will be made available.'),
    '#type' => 'checkboxes',
    '#required' => TRUE,
    '#options' => $entity_options,
    '#default_value' => $view_mode->entities,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save')
  );

  return $form;
}

/**
 * Return whether a view mode machine name is unique.
 */
function ds_view_mode_unique($name) {
  $custom_view_modes = ds_custom_view_modes();
  $value = strtr($name, array('-' => '_'));
  return isset($custom_view_modes[$value]) ? TRUE : FALSE;
}

/**
 * Validates the view mode submission form generated by ds_edit_view_mode_form().
 */
function ds_edit_view_mode_form_validate($form, &$form_state) {

  $view_mode = new stdClass;
  $view_mode->view_mode = $form_state['values']['view_mode'];
  $view_mode->label = $form_state['values']['name'];

  $entities = $form_state['values']['entities'];
  foreach ($entities as $key => $value) {
    if ($key !== $value) {
      unset($entities[$key]);
    }
  }
  $view_mode->entities = serialize($entities);

  $reserved_view_modes = array('form', 'default', 'full', 'teaser', 'search_result', 'search_index', 'rss');
  if (in_array($view_mode->view_mode, $reserved_view_modes)) {
    form_set_error('type', t('The machine-readable name %view_mode is reserved.', array('%view_mode' => $view_mode->view_mode)));
  }

  $form_state['view_mode'] = $view_mode;
}

/**
 * Saves the view mode.
 */
function ds_edit_view_mode_form_submit($form, &$form_state) {
  $view_mode = $form_state['view_mode'];

  // Delete previous view_mode configuration.
  db_delete('ds_view_modes')
    ->condition('view_mode', $view_mode->view_mode)
    ->execute();

  // Save the new view mode.
  drupal_write_record('ds_view_modes', $view_mode);

  // Clear entity info cache and trigger menu build on next request.
  cache_clear_all('entity_info', 'cache', TRUE);
  variable_set('menu_rebuild_needed', TRUE);

  // Redirect.
  $form_state['redirect'] = 'admin/structure/ds/view_modes';
  drupal_set_message(t('The view mode %view_mode has been saved', array('%view_mode' => $view_mode->label)));
}

/**
 * Confirmation view mode delete form
 */
function ds_delete_view_mode_confirm($form, &$form_state, $view_mode = '') {
  $custom_view_modes = ds_custom_view_modes();
  if (isset($custom_view_modes[$view_mode])) {
    $view_mode = $custom_view_modes[$view_mode];
    $form['#view_mode'] = $view_mode;
    return confirm_form($form,
      t('Are you sure you want to delete %view_mode?', array('%view_mode' => $view_mode->label)),
      'admin/structure/ds/view_modes',
      t('This action cannot be undone.'),
      t('Delete'),
      t('Cancel')
    );
  }
  else {
    drupal_set_message(t('Unknown view mode'));
    drupal_goto('admin/structure/ds/view_modes');
  }
}

/**
 * Confirmed view mode delete submit callback.
 */
function ds_delete_view_mode_confirm_submit($form, &$form_state) {

  $view_mode = $form['#view_mode'];

  // Remove view mode.
  db_delete('ds_view_modes')
    ->condition('view_mode', $view_mode->view_mode)
    ->execute();

  // @todo remove any stale data (ds_settings).

  // Clear entity info cache and trigger menu build on next request.
  cache_clear_all('entity_info', 'cache', TRUE);
  variable_set('menu_rebuild_needed', TRUE);

  // Redirect.
  $form_state['redirect'] = 'admin/structure/ds/view_modes';
  drupal_set_message(t('The view mode %view_mode has been deleted', array('%view_mode' => $view_mode->label)));
}
